@@ -4,37 +4,37 @@ drawing legends associated with axes and/or figures.
 
 .. important::
 
-    It is unlikely that you would ever create a Legend instance manually.
-    Most users would normally create a legend via the
+    It is unlikely that you would ever create a Legend instance
+    manually.  Most users would normally create a legend via the
     :meth:`~matplotlib.axes.Axes.legend` function. For more details on legends
-    there is also a :ref:`legend guide
-    <sphx_glr_tutorials_intermediate_legend_guide.py>`.
-
-The Legend class can be considered as a container of legend handles
-and legend texts. Creation of corresponding legend handles from the
-plot elements in the axes or figures (e.g., lines, patches, etc.) are
-specified by the handler map, which defines the mapping between the
-plot elements and the legend handlers to be used (the default legend
-handlers are defined in the :mod:`~matplotlib.legend_handler` module).
-Note that not all kinds of artist are supported by the legend yet by default
-but it is possible to extend the legend handler's capabilities to support
-arbitrary objects. See the :ref:`legend guide
-<sphx_glr_tutorials_intermediate_legend_guide.py>` for more information.
+    there is also a :doc:`legend guide </tutorials/intermediate/legend_guide>`.
 
+The Legend class can be considered as a container of legend handles and
+legend texts. Creation of corresponding legend handles from the plot elements
+in the axes or figures (e.g., lines, patches, etc.) are specified by the
+handler map, which defines the mapping between the plot elements and the
+legend handlers to be used (the default legend handlers are defined in the
+:mod:`~matplotlib.legend_handler` module).  Note that not all kinds of
+artist are supported by the legend yet by default but it is possible to
+extend the legend handler's capabilities to support arbitrary objects. See
+the :doc:`legend guide </tutorials/intermediate/legend_guide>` for more
+information.
 """
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
 import six
-from six.moves import xrange
 
+import logging
 import warnings
 
 import numpy as np
 
 from matplotlib import rcParams
+from matplotlib import docstring
 from matplotlib.artist import Artist, allow_rasterization
 from matplotlib.cbook import silent_list, is_hashable
+import matplotlib.colors as colors
 from matplotlib.font_manager import FontProperties
 from matplotlib.lines import Line2D
 from matplotlib.patches import Patch, Rectangle, Shadow, FancyBboxPatch
@@ -54,9 +54,11 @@ from . import legend_handler
 class DraggableLegend(DraggableOffsetBox):
     def __init__(self, legend, use_blit=False, update="loc"):
         """
-        update : If "loc", update *loc* parameter of
-                 legend upon finalizing. If "bbox", update
-                 *bbox_to_anchor* parameter.
+        Parameters
+        ----------
+        update : string
+            If "loc", update *loc* parameter of legend upon finalizing.
+            If "bbox", update *bbox_to_anchor* parameter.
         """
         self.legend = legend
 
@@ -105,28 +107,182 @@ class DraggableLegend(DraggableOffsetBox):
         self.legend.set_bbox_to_anchor(loc_in_bbox)
 
 
+_legend_kw_doc = '''
+loc : int or string or pair of floats, default: :rc:`legend.loc` ('best' for \
+axes, 'upper right' for figures)
+    The location of the legend. Possible codes are:
+
+        ===============   =============
+        Location String   Location Code
+        ===============   =============
+        'best'            0
+        'upper right'     1
+        'upper left'      2
+        'lower left'      3
+        'lower right'     4
+        'right'           5
+        'center left'     6
+        'center right'    7
+        'lower center'    8
+        'upper center'    9
+        'center'          10
+        ===============   =============
+
+
+    Alternatively can be a 2-tuple giving ``x, y`` of the lower-left
+    corner of the legend in axes coordinates (in which case
+    ``bbox_to_anchor`` will be ignored).
+
+bbox_to_anchor : `.BboxBase` or pair of floats
+    Specify any arbitrary location for the legend in `bbox_transform`
+    coordinates (default Axes coordinates).
+
+    For example, to put the legend's upper right hand corner in the
+    center of the axes the following keywords can be used::
+
+        loc='upper right', bbox_to_anchor=(0.5, 0.5)
+
+ncol : integer
+    The number of columns that the legend has. Default is 1.
+
+prop : None or :class:`matplotlib.font_manager.FontProperties` or dict
+    The font properties of the legend. If None (default), the current
+    :data:`matplotlib.rcParams` will be used.
+
+fontsize : int or float or {'xx-small', 'x-small', 'small', 'medium', \
+'large', 'x-large', 'xx-large'}
+    Controls the font size of the legend. If the value is numeric the
+    size will be the absolute font size in points. String values are
+    relative to the current default font size. This argument is only
+    used if `prop` is not specified.
+
+numpoints : None or int
+    The number of marker points in the legend when creating a legend
+    entry for a `.Line2D` (line).
+    Default is ``None``, which will take the value from
+    :rc:`legend.numpoints`.
+
+scatterpoints : None or int
+    The number of marker points in the legend when creating
+    a legend entry for a `.PathCollection` (scatter plot).
+    Default is ``None``, which will take the value from
+    :rc:`legend.scatterpoints`.
+
+scatteryoffsets : iterable of floats
+    The vertical offset (relative to the font size) for the markers
+    created for a scatter plot legend entry. 0.0 is at the base the
+    legend text, and 1.0 is at the top. To draw all markers at the
+    same height, set to ``[0.5]``. Default is ``[0.375, 0.5, 0.3125]``.
+
+markerscale : None or int or float
+    The relative size of legend markers compared with the originally
+    drawn ones.
+    Default is ``None``, which will take the value from
+    :rc:`legend.markerscale`.
+
+markerfirst : bool
+    If *True*, legend marker is placed to the left of the legend label.
+    If *False*, legend marker is placed to the right of the legend
+    label.
+    Default is *True*.
+
+frameon : None or bool
+    Control whether the legend should be drawn on a patch
+    (frame).
+    Default is ``None``, which will take the value from
+    :rc:`legend.frameon`.
+
+fancybox : None or bool
+    Control whether round edges should be enabled around the
+    :class:`~matplotlib.patches.FancyBboxPatch` which makes up the
+    legend's background.
+    Default is ``None``, which will take the value from
+    :rc:`legend.fancybox`.
+
+shadow : None or bool
+    Control whether to draw a shadow behind the legend.
+    Default is ``None``, which will take the value from
+    :rc:`legend.shadow`.
+
+framealpha : None or float
+    Control the alpha transparency of the legend's background.
+    Default is ``None``, which will take the value from
+    :rc:`legend.framealpha`.  If shadow is activated and
+    *framealpha* is ``None``, the default value is ignored.
+
+facecolor : None or "inherit" or a color spec
+    Control the legend's background color.
+    Default is ``None``, which will take the value from
+    :rc:`legend.facecolor`.  If ``"inherit"``, it will take
+    :rc:`axes.facecolor`.
+
+edgecolor : None or "inherit" or a color spec
+    Control the legend's background patch edge color.
+    Default is ``None``, which will take the value from
+    :rc:`legend.edgecolor` If ``"inherit"``, it will take
+    :rc:`axes.edgecolor`.
+
+mode : {"expand", None}
+    If `mode` is set to ``"expand"`` the legend will be horizontally
+    expanded to fill the axes area (or `bbox_to_anchor` if defines
+    the legend's size).
+
+bbox_transform : None or :class:`matplotlib.transforms.Transform`
+    The transform for the bounding box (`bbox_to_anchor`). For a value
+    of ``None`` (default) the Axes'
+    :data:`~matplotlib.axes.Axes.transAxes` transform will be used.
+
+title : str or None
+    The legend's title. Default is no title (``None``).
+
+borderpad : float or None
+    The fractional whitespace inside the legend border.
+    Measured in font-size units.
+    Default is ``None``, which will take the value from
+    :rc:`legend.borderpad`.
+
+labelspacing : float or None
+    The vertical space between the legend entries.
+    Measured in font-size units.
+    Default is ``None``, which will take the value from
+    :rc:`legend.labelspacing`.
+
+handlelength : float or None
+    The length of the legend handles.
+    Measured in font-size units.
+    Default is ``None``, which will take the value from
+    :rc:`legend.handlelength`.
+
+handletextpad : float or None
+    The pad between the legend handle and text.
+    Measured in font-size units.
+    Default is ``None``, which will take the value from
+    :rc:`legend.handletextpad`.
+
+borderaxespad : float or None
+    The pad between the axes and legend border.
+    Measured in font-size units.
+    Default is ``None``, which will take the value from
+    :rc:`legend.borderaxespad`.
+
+columnspacing : float or None
+    The spacing between columns.
+    Measured in font-size units.
+    Default is ``None``, which will take the value from
+    :rc:`legend.columnspacing`.
+
+handler_map : dict or None
+    The custom dictionary mapping instances or types to a legend
+    handler. This `handler_map` updates the default handler map
+    found at :func:`matplotlib.legend.Legend.get_legend_handler_map`.
+
+'''
+docstring.interpd.update(_legend_kw_doc=_legend_kw_doc)
+
+
 class Legend(Artist):
     """
-    Place a legend on the axes at location loc.  Labels are a
-    sequence of strings and loc can be a string or an integer
-    specifying the legend location
-
-    The location codes are::
-
-      'best'         : 0, (only implemented for axes legends)
-      'upper right'  : 1,
-      'upper left'   : 2,
-      'lower left'   : 3,
-      'lower right'  : 4,
-      'right'        : 5, (same as 'center right', for back-compatibility)
-      'center left'  : 6,
-      'center right' : 7,
-      'lower center' : 8,
-      'upper center' : 9,
-      'center'       : 10,
-
-    loc can be a tuple of the normalized coordinate values with
-    respect its parent.
+    Place a legend on the axes at location loc.
 
     """
     codes = {'best':         0,  # only implemented for axes legends
@@ -147,6 +303,7 @@ class Legend(Artist):
     def __str__(self):
         return "Legend"
 
+    @docstring.dedent_interpd
     def __init__(self, parent, handles, labels,
                  loc=None,
                  numpoints=None,    # the number of points in the legend line
@@ -190,47 +347,192 @@ class Legend(Artist):
                  handler_map=None,
                  ):
         """
-        - *parent*: the artist that contains the legend
-        - *handles*: a list of artists (lines, patches) to be added to the
-                      legend
-        - *labels*: a list of strings to label the legend
 
-        Optional keyword arguments:
+        Parameters
+        ----------
+        parent : `~matplotlib.axes.Axes` or `.Figure`
+            The artist that contains the legend.
 
-        ================   ====================================================
-        Keyword            Description
-        ================   ====================================================
-        loc                Location code string, or tuple (see below).
-        prop               the font property
-        fontsize           the font size (used only if prop is not specified)
-        markerscale        the relative size of legend markers vs. original
-        markerfirst        If True (default), marker is to left of the label.
-        numpoints          the number of points in the legend for line
-        scatterpoints      the number of points in the legend for scatter plot
-        scatteryoffsets    a list of yoffsets for scatter symbols in legend
-        frameon            If True, draw the legend on a patch (frame).
-        fancybox           If True, draw the frame with a round fancybox.
-        shadow             If True, draw a shadow behind legend.
-        framealpha         Transparency of the frame.
-        edgecolor          Frame edgecolor.
-        facecolor          Frame facecolor.
-        ncol               number of columns
-        borderpad          the fractional whitespace inside the legend border
-        labelspacing       the vertical space between the legend entries
-        handlelength       the length of the legend handles
-        handleheight       the height of the legend handles
-        handletextpad      the pad between the legend handle and text
-        borderaxespad      the pad between the axes and legend border
-        columnspacing      the spacing between columns
-        title              the legend title
-        bbox_to_anchor     the bbox that the legend will be anchored.
-        bbox_transform     the transform for the bbox. transAxes if None.
-        ================   ====================================================
+        handles : sequence of `.Artist`
+            A list of Artists (lines, patches) to be added to the legend.
+
+        labels : sequence of strings
+            A list of labels to show next to the artists. The length of handles
+            and labels should be the same. If they are not, they are truncated
+            to the smaller of both lengths.
+
+        Other Parameters
+        ----------------
+
+        loc : int or string or pair of floats, default: 'upper right'
+            The location of the legend. Possible codes are:
+
+                ===============   =============
+                Location String   Location Code
+                ===============   =============
+                'best'            0
+                'upper right'     1
+                'upper left'      2
+                'lower left'      3
+                'lower right'     4
+                'right'           5
+                'center left'     6
+                'center right'    7
+                'lower center'    8
+                'upper center'    9
+                'center'          10
+                ===============   =============
 
 
-        The pad and spacing parameters are measured in font-size units.  e.g.,
-        a fontsize of 10 points and a handlelength=5 implies a handlelength of
-        50 points.  Values from rcParams will be used if None.
+            Alternatively can be a 2-tuple giving ``x, y`` of the lower-left
+            corner of the legend in axes coordinates (in which case
+            ``bbox_to_anchor`` will be ignored).
+
+        bbox_to_anchor : `.BboxBase` or pair of floats
+            Specify any arbitrary location for the legend in `bbox_transform`
+            coordinates (default Axes coordinates).
+
+            For example, to put the legend's upper right hand corner in the
+            center of the axes the following keywords can be used::
+
+               loc='upper right', bbox_to_anchor=(0.5, 0.5)
+
+        ncol : integer
+            The number of columns that the legend has. Default is 1.
+
+        prop : None or :class:`matplotlib.font_manager.FontProperties` or dict
+            The font properties of the legend. If None (default), the current
+            :data:`matplotlib.rcParams` will be used.
+
+        fontsize : int or float or {'xx-small', 'x-small', 'small', 'medium', \
+'large', 'x-large', 'xx-large'}
+            Controls the font size of the legend. If the value is numeric the
+            size will be the absolute font size in points. String values are
+            relative to the current default font size. This argument is only
+            used if `prop` is not specified.
+
+        numpoints : None or int
+            The number of marker points in the legend when creating a legend
+            entry for a `.Line2D` (line).
+            Default is ``None``, which will take the value from
+            :rc:`legend.numpoints`.
+
+        scatterpoints : None or int
+            The number of marker points in the legend when creating
+            a legend entry for a `.PathCollection` (scatter plot).
+            Default is ``None``, which will take the value from
+            :rc:`legend.scatterpoints`.
+
+        scatteryoffsets : iterable of floats
+            The vertical offset (relative to the font size) for the markers
+            created for a scatter plot legend entry. 0.0 is at the base the
+            legend text, and 1.0 is at the top. To draw all markers at the
+            same height, set to ``[0.5]``. Default is ``[0.375, 0.5, 0.3125]``.
+
+        markerscale : None or int or float
+            The relative size of legend markers compared with the originally
+            drawn ones.
+            Default is ``None``, which will take the value from
+            :rc:`legend.markerscale`.
+
+        markerfirst : bool
+            If *True*, legend marker is placed to the left of the legend label.
+            If *False*, legend marker is placed to the right of the legend
+            label.
+            Default is *True*.
+
+        frameon : None or bool
+            Control whether the legend should be drawn on a patch
+            (frame).
+            Default is ``None``, which will take the value from
+            :rc:`legend.frameon`.
+
+        fancybox : None or bool
+            Control whether round edges should be enabled around the
+            :class:`~matplotlib.patches.FancyBboxPatch` which makes up the
+            legend's background.
+            Default is ``None``, which will take the value from
+            :rc:`legend.fancybox`.
+
+        shadow : None or bool
+            Control whether to draw a shadow behind the legend.
+            Default is ``None``, which will take the value from
+            :rc:`legend.shadow`.
+
+        framealpha : None or float
+            Control the alpha transparency of the legend's background.
+            Default is ``None``, which will take the value from
+            :rc:`legend.framealpha`.  If shadow is activated and
+            *framealpha* is ``None``, the default value is ignored.
+
+        facecolor : None or "inherit" or a color spec
+            Control the legend's background color.
+            Default is ``None``, which will take the value from
+            :rc:`legend.facecolor`.  If ``"inherit"``, it will take
+            :rc:`axes.facecolor`.
+
+        edgecolor : None or "inherit" or a color spec
+            Control the legend's background patch edge color.
+            Default is ``None``, which will take the value from
+            :rc:`legend.edgecolor` If ``"inherit"``, it will take
+            :rc:`axes.edgecolor`.
+
+        mode : {"expand", None}
+            If `mode` is set to ``"expand"`` the legend will be horizontally
+            expanded to fill the axes area (or `bbox_to_anchor` if defines
+            the legend's size).
+
+        bbox_transform : None or :class:`matplotlib.transforms.Transform`
+            The transform for the bounding box (`bbox_to_anchor`). For a value
+            of ``None`` (default) the Axes'
+            :data:`~matplotlib.axes.Axes.transAxes` transform will be used.
+
+        title : str or None
+            The legend's title. Default is no title (``None``).
+
+        borderpad : float or None
+            The fractional whitespace inside the legend border.
+            Measured in font-size units.
+            Default is ``None``, which will take the value from
+            :rc:`legend.borderpad`.
+
+        labelspacing : float or None
+            The vertical space between the legend entries.
+            Measured in font-size units.
+            Default is ``None``, which will take the value from
+            :rc:`legend.labelspacing`.
+
+        handlelength : float or None
+            The length of the legend handles.
+            Measured in font-size units.
+            Default is ``None``, which will take the value from
+            :rc:`legend.handlelength`.
+
+        handletextpad : float or None
+            The pad between the legend handle and text.
+            Measured in font-size units.
+            Default is ``None``, which will take the value from
+            :rc:`legend.handletextpad`.
+
+        borderaxespad : float or None
+            The pad between the axes and legend border.
+            Measured in font-size units.
+            Default is ``None``, which will take the value from
+            :rc:`legend.borderaxespad`.
+
+        columnspacing : float or None
+            The spacing between columns.
+            Measured in font-size units.
+            Default is ``None``, which will take the value from
+            :rc:`legend.columnspacing`.
+
+        handler_map : dict or None
+            The custom dictionary mapping instances or types to a legend
+            handler. This `handler_map` updates the default handler map
+            found at :func:`matplotlib.legend.Legend.get_legend_handler_map`.
+
+        Notes
+        -----
 
         Users can specify any arbitrary location for the legend using the
         *bbox_to_anchor* keyword argument. bbox_to_anchor can be an instance
@@ -280,6 +582,18 @@ class Legend(Artist):
                 value = locals_view[name]
             setattr(self, name, value)
         del locals_view
+        # trim handles and labels if illegal label...
+        _lab, _hand = [], []
+        for label, handle in zip(labels, handles):
+            if (isinstance(label, six.string_types) and
+                    label.startswith('_')):
+                warnings.warn('The handle {!r} has a label of {!r} which '
+                              'cannot be automatically added to the '
+                              'legend.'.format(handle, label))
+            else:
+                _lab.append(label)
+                _hand.append(handle)
+        labels, handles = _lab, _hand
 
         handles = list(handles)
         if len(handles) < 2:
@@ -401,7 +715,7 @@ class Legend(Artist):
 
     def _set_artist_props(self, a):
         """
-        set the boilerplate props for artists added to axes
+        Set the boilerplate props for artists added to axes.
         """
         a.set_figure(self.figure)
         if self.isaxes:
@@ -416,6 +730,7 @@ class Legend(Artist):
         # value of the find_offset.
         self._loc_real = loc
         self.stale = True
+        self._legend_box.set_offset(self._findoffset)
 
     def _get_loc(self):
         return self._loc_real
@@ -423,7 +738,7 @@ class Legend(Artist):
     _loc = property(_get_loc, _set_loc)
 
     def _findoffset(self, width, height, xdescent, ydescent, renderer):
-        "Helper function to locate the legend"
+        "Helper function to locate the legend."
 
         if self._loc == 0:  # "best".
             x, y = self._find_best_position(width, height, renderer)
@@ -441,7 +756,7 @@ class Legend(Artist):
 
     @allow_rasterization
     def draw(self, renderer):
-        "Draw everything that belongs to the legend"
+        "Draw everything that belongs to the legend."
         if not self.get_visible():
             return
 
@@ -528,7 +843,7 @@ class Legend(Artist):
 
     def get_legend_handler_map(self):
         """
-        return the handler map.
+        Return the handler map.
         """
 
         default_handler_map = self.get_default_handler_map()
@@ -543,7 +858,7 @@ class Legend(Artist):
     @staticmethod
     def get_legend_handler(legend_handler_map, orig_handle):
         """
-        return a legend handler from *legend_handler_map* that
+        Return a legend handler from *legend_handler_map* that
         corresponds to *orig_handler*.
 
         *legend_handler_map* should be a dictionary object (that is
@@ -553,7 +868,7 @@ class Legend(Artist):
         *legend_hanler_map* and return the associated value.
         Otherwise, it checks for each of the classes in its
         method-resolution-order. If no matching key is found, it
-        returns None.
+        returns ``None``.
         """
         if is_hashable(orig_handle):
             try:
@@ -589,15 +904,13 @@ class Legend(Artist):
 
         text_list = []  # the list of text instances
         handle_list = []  # the list of text instances
+        handles_and_labels = []
 
         label_prop = dict(verticalalignment='baseline',
                           horizontalalignment='left',
                           fontproperties=self.prop,
                           )
 
-        labelboxes = []
-        handleboxes = []
-
         # The approximate height and descent of text. These values are
         # only used for plotting the legend handle.
         descent = 0.35 * self._approx_text_height() * (self.handleheight - 0.7)
@@ -620,7 +933,8 @@ class Legend(Artist):
                     "Legend does not support {!r} instances.\nA proxy artist "
                     "may be used instead.\nSee: "
                     "http://matplotlib.org/users/legend_guide.html"
-                    "#using-proxy-artist".format(orig_handle)
+                    "#creating-artists-specifically-for-adding-to-the-legend-"
+                    "aka-proxy-artists".format(orig_handle)
                 )
                 # We don't have a handle for this artist, so we just defer
                 # to None.
@@ -629,26 +943,23 @@ class Legend(Artist):
                 textbox = TextArea(lab, textprops=label_prop,
                                    multilinebaseline=True,
                                    minimumdescent=True)
-                text_list.append(textbox._text)
-
-                labelboxes.append(textbox)
-
                 handlebox = DrawingArea(width=self.handlelength * fontsize,
                                         height=height,
                                         xdescent=0., ydescent=descent)
-                handleboxes.append(handlebox)
 
+                text_list.append(textbox._text)
                 # Create the artist for the legend which represents the
                 # original artist/handle.
                 handle_list.append(handler.legend_artist(self, orig_handle,
                                                          fontsize, handlebox))
+                handles_and_labels.append((handlebox, textbox))
 
-        if handleboxes:
+        if handles_and_labels:
             # We calculate number of rows in each column. The first
             # (num_largecol) columns will have (nrows+1) rows, and remaining
             # (num_smallcol) columns will have (nrows) rows.
-            ncol = min(self._ncol, len(handleboxes))
-            nrows, num_largecol = divmod(len(handleboxes), ncol)
+            ncol = min(self._ncol, len(handles_and_labels))
+            nrows, num_largecol = divmod(len(handles_and_labels), ncol)
             num_smallcol = ncol - num_largecol
             # starting index of each column and number of rows in it.
             rows_per_col = [nrows + 1] * num_largecol + [nrows] * num_smallcol
@@ -657,7 +968,6 @@ class Legend(Artist):
         else:
             cols = []
 
-        handle_label = list(zip(handleboxes, labelboxes))
         columnbox = []
         for i0, di in cols:
             # pack handleBox and labelBox into itemBox
@@ -665,7 +975,7 @@ class Legend(Artist):
                                  sep=self.handletextpad * fontsize,
                                  children=[h, t] if markerfirst else [t, h],
                                  align="baseline")
-                         for h, t in handle_label[i0:i0 + di]]
+                         for h, t in handles_and_labels[i0:i0 + di]]
             # minimumdescent=False for the text of the last row of the column
             if markerfirst:
                 itemBoxes[-1].get_children()[1].set_minimumdescent(False)
@@ -692,7 +1002,6 @@ class Legend(Artist):
                                    children=[self._legend_title_box,
                                              self._legend_handle_box])
         self._legend_box.set_figure(self.figure)
-        self._legend_box.set_offset(self._findoffset)
         self.texts = text_list
         self.legendHandles = handle_list
 
@@ -749,11 +1058,17 @@ class Legend(Artist):
         return [vertices, bboxes, lines, offsets]
 
     def draw_frame(self, b):
-        'b is a boolean.  Set draw frame to b'
+        '''
+        Set draw frame to b.
+
+        Parameters
+        ----------
+        b : bool
+        '''
         self.set_frame_on(b)
 
     def get_children(self):
-        'return a list of child artists'
+        'Return a list of child artists.'
         children = []
         if self._legend_box:
             children.append(self._legend_box)
@@ -762,26 +1077,28 @@ class Legend(Artist):
         return children
 
     def get_frame(self):
-        'return the Rectangle instance used to frame the legend'
+        '''
+        Return the `~.patches.Rectangle` instances used to frame the legend.
+        '''
         return self.legendPatch
 
     def get_lines(self):
-        'return a list of lines.Line2D instances in the legend'
+        'Return a list of `~.lines.Line2D` instances in the legend.'
         return [h for h in self.legendHandles if isinstance(h, Line2D)]
 
     def get_patches(self):
-        'return a list of patch instances in the legend'
+        'Return a list of `~.patches.Patch` instances in the legend.'
         return silent_list('Patch',
                            [h for h in self.legendHandles
                             if isinstance(h, Patch)])
 
     def get_texts(self):
-        'return a list of text.Text instance in the legend'
+        'Return a list of `~.text.Text` instances in the legend.'
         return silent_list('Text', self.texts)
 
     def set_title(self, title, prop=None):
         """
-        set the legend title. Fontproperties can be optionally set
+        Set the legend title. Fontproperties can be optionally set
         with *prop* parameter.
         """
         self._legend_title_box._text.set_text(title)
@@ -798,32 +1115,31 @@ class Legend(Artist):
         self.stale = True
 
     def get_title(self):
-        'return Text instance for the legend title'
+        'Return the `.Text` instance for the legend title.'
         return self._legend_title_box._text
 
     def get_window_extent(self, *args, **kwargs):
-        'return a extent of the legend'
+        'Return extent of the legend.'
         return self.legendPatch.get_window_extent(*args, **kwargs)
 
     def get_frame_on(self):
-        """
-        Get whether the legend box patch is drawn
-        """
+        """Get whether the legend box patch is drawn."""
         return self._drawFrame
 
     def set_frame_on(self, b):
         """
-        Set whether the legend box patch is drawn
+        Set whether the legend box patch is drawn.
 
-        ACCEPTS: [ *True* | *False* ]
+        Parameters
+        ----------
+        b : bool
+            .. ACCEPTS: bool
         """
         self._drawFrame = b
         self.stale = True
 
     def get_bbox_to_anchor(self):
-        """
-        return the bbox that the legend will be anchored
-        """
+        """Return the bbox that the legend will be anchored to."""
         if self._bbox_to_anchor is None:
             return self.parent.bbox
         else:
@@ -831,12 +1147,15 @@ class Legend(Artist):
 
     def set_bbox_to_anchor(self, bbox, transform=None):
         """
-        set the bbox that the legend will be anchored.
+        Set the bbox that the legend will be anchored to.
 
-        *bbox* can be a BboxBase instance, a tuple of [left, bottom,
-        width, height] in the given transform (normalized axes
-        coordinate if None), or a tuple of [left, bottom] where the
-        width and height will be assumed to be zero.
+        *bbox* can be
+
+        - A `.BboxBase` instance
+        - A tuple of ``(left, bottom, width, height)`` in the given transform
+          (normalized axes coordinate if None)
+        - A tuple of ``(left, bottom)`` where the width and height will be
+          assumed to be zero.
         """
         if bbox is None:
             self._bbox_to_anchor = None
@@ -870,13 +1189,13 @@ class Legend(Artist):
           This corresponds to the possible values for self._loc, excluding
           "best".
 
-        - bbox: bbox to be placed, display coodinate units.
+        - bbox: bbox to be placed, display coordinate units.
         - parentbbox: a parent box which will contain the bbox. In
             display coordinates.
         """
         assert loc in range(1, 11)  # called only internally
 
-        BEST, UR, UL, LL, LR, R, CL, CR, LC, UC, C = list(xrange(11))
+        BEST, UR, UL, LL, LR, R, CL, CR, LC, UC, C = range(11)
 
         anchor_coefs = {UR: "NE",
                         UL: "NW",
@@ -900,7 +1219,7 @@ class Legend(Artist):
         """
         Determine the best location to place the legend.
 
-        `consider` is a list of (x, y) pairs to consider as a potential
+        *consider* is a list of ``(x, y)`` pairs to consider as a potential
         lower-left corner of the legend. All are display coords.
         """
         # should always hold because function is only called internally
@@ -949,7 +1268,7 @@ class Legend(Artist):
           * False : turn draggable off
 
         If draggable is on, you can drag the legend on the canvas with
-        the mouse.  The DraggableLegend helper instance is returned if
+        the mouse. The `.DraggableLegend` helper instance is returned if
         draggable is on.
 
         The update parameter control which parameter of the legend changes
@@ -973,3 +1292,110 @@ class Legend(Artist):
             self._draggable = None
 
         return self._draggable
+
+
+# Helper functions to parse legend arguments for both `figure.legend` and
+# `axes.legend`:
+def _get_legend_handles(axs, legend_handler_map=None):
+    """
+    Return a generator of artists that can be used as handles in
+    a legend.
+
+    """
+    handles_original = []
+    for ax in axs:
+        handles_original += (ax.lines + ax.patches +
+                             ax.collections + ax.containers)
+        # support parasite axes:
+        if hasattr(ax, 'parasites'):
+            for axx in ax.parasites:
+                handles_original += (axx.lines + axx.patches +
+                                     axx.collections + axx.containers)
+
+    handler_map = Legend.get_default_handler_map()
+
+    if legend_handler_map is not None:
+        handler_map = handler_map.copy()
+        handler_map.update(legend_handler_map)
+
+    has_handler = Legend.get_legend_handler
+
+    for handle in handles_original:
+        label = handle.get_label()
+        if label != '_nolegend_' and has_handler(handler_map, handle):
+            yield handle
+
+
+def _get_legend_handles_labels(axs, legend_handler_map=None):
+    """
+    Return handles and labels for legend, internal method.
+
+    """
+    handles = []
+    labels = []
+
+    for handle in _get_legend_handles(axs, legend_handler_map):
+        label = handle.get_label()
+        if (label and not label.startswith('_')):
+            handles.append(handle)
+            labels.append(label)
+    return handles, labels
+
+
+def _parse_legend_args(axs, *args, **kwargs):
+    """
+    Get the handles and labels from the calls to either ``figure.legend``
+    or ``axes.legend``.
+
+    ``axs`` is a list of axes (to get legend artists from)
+    """
+    log = logging.getLogger(__name__)
+
+    handlers = kwargs.get('handler_map', {}) or {}
+
+    # Support handles and labels being passed as keywords.
+    handles = kwargs.pop('handles', None)
+    labels = kwargs.pop('labels', None)
+
+    extra_args = ()
+
+    if (handles is not None or labels is not None) and len(args):
+        warnings.warn("You have mixed positional and keyword "
+                      "arguments, some input may be "
+                      "discarded.")
+
+    # if got both handles and labels as kwargs, make same length
+    if handles and labels:
+        handles, labels = zip(*zip(handles, labels))
+
+    elif handles is not None and labels is None:
+        labels = [handle.get_label() for handle in handles]
+
+    elif labels is not None and handles is None:
+        # Get as many handles as there are labels.
+        handles = [handle for handle, label
+                   in zip(_get_legend_handles(axs, handlers), labels)]
+
+    # No arguments - automatically detect labels and handles.
+    elif len(args) == 0:
+        handles, labels = _get_legend_handles_labels(axs, handlers)
+        if not handles:
+            log.warning('No handles with labels found to put in legend.')
+
+    # One argument. User defined labels - automatic handle detection.
+    elif len(args) == 1:
+        labels, = args
+        # Get as many handles as there are labels.
+        handles = [handle for handle, label
+                   in zip(_get_legend_handles(axs, handlers), labels)]
+
+    # Two arguments:
+    #   * user defined handles and labels
+    elif len(args) >= 2:
+        handles, labels = args[:2]
+        extra_args = args[2:]
+
+    else:
+        raise TypeError('Invalid arguments to legend.')
+
+    return handles, labels, extra_args, kwargs
